using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Data.Linq;
using System.Data.Linq.Mapping;
using System.Linq;
using System.Reflection;
using Magiq.Providers.Sql;
using Magiq.Tests.Sql.TestModel;
using Magiq.Tests.Support;

namespace Magiq.Tests.Sql
{
    public class MagiqToSqlTestConfigurator : IMagiqTestConfigurator
    {
        private static readonly DataLoadOptions LoadOptions;

        public DataContext DataContext { get; private set; }

        static MagiqToSqlTestConfigurator()
        {
            Current = new MagiqToSqlTestConfigurator();
            LoadOptions = new DataLoadOptions();
            LoadOptions.LoadWith<LinqToSqlTestClass>(x => x.Reference);
            LoadOptions.LoadWith<LinqToSqlTestClass>(x => x.Reference2);
            LoadOptions.LoadWith<CompositePkParent>(x => x.Child);

        }

        private MagiqToSqlTestConfigurator()
        {
        }

        public static MagiqToSqlTestConfigurator Current { get; private set; }

        #region IMagiqTestConfigurator Members

        public void Persist<T>(IList<T> items) where T : class
        {
            DataContext.GetTable<T>().InsertAllOnSubmit(items);
            DataContext.SubmitChanges();
        }

        public void SetUp()
        {
            MagiqConfigurator.Clear()
                .Register<MagiqMock>()
                .Register<MagiqToSql>();

            CreateDataContext();

            if (DataContext.DatabaseExists())
                DataContext.DeleteDatabase();
            DataContext.CreateDatabase();
        }

        public void TearDown()
        {
            CloseDataContext();
        }

        public IQueryable<T> GetQueryable<T>() where T : class
        {
            return DataContext.GetTable<T>();
        }
        
        public void Reload()
        {
            DataContext.SubmitChanges();
            CloseDataContext();
            CreateDataContext();
        }

        #endregion

        private static readonly MappingSource MappingSource = CreateMappingSource();

        private static MappingSource CreateMappingSource()
        {
            var mapping = Assembly.GetExecutingAssembly().GetManifestResourceStream("Magiq.Tests.Sql.Mappings.xml");
            return XmlMappingSource.FromStream(mapping);
        }

        private void CreateDataContext()
        {
            
            DataContext = new DataContext(ConfigurationManager.ConnectionStrings["SqlServer"].ConnectionString,
                                          MappingSource)

                              {
                                  DeferredLoadingEnabled = true,
                                  LoadOptions = LoadOptions
                              };
        }

        private void CloseDataContext()
        {
            var c = DataContext.Connection;
            if (c.State != ConnectionState.Closed)
                c.Close();
            DataContext.Dispose();
            c.Dispose();
        }
    }
}