using System;
using System.Collections.Generic;
using System.Linq;
using Magiq.Support;
using Magiq.Update;
using MirrorMirror;

namespace Magiq.Providers.Objects
{
    public class UpdateStrategy<T> : IUpdateStrategy where T : class
    {
        private readonly Dictionary<IPropertySet, Delegate> compiledChains = new Dictionary<IPropertySet, Delegate>();

        private readonly Dictionary<IPropertySet, Delegate> compiledSettersValues =
            new Dictionary<IPropertySet, Delegate>();

        private readonly IUpdate<T> update;

        public UpdateStrategy(IUpdate<T> update)
        {
            this.update = update;
        }

        #region IUpdateStrategy Members

        public int Execute()
        {
            CompileProperties();

            var updates = new List<object>();

            foreach (var item in update.Source)
            {
                foreach (var set in update.PropertySets)
                {
                    var returnType = set.ValueExpression.Body.Type;

                    object value;

                    var candidateValue = compiledSettersValues[set].DynamicInvoke(item);

                    if (set.PropertyType == returnType)
                    {
                        try
                        {
                            value = candidateValue;
                        }
                        catch (NullReferenceException)
                        {
                            value = null;
                        }
                    }
                    else
                    {
                        if (set.PropertyType.IsGenericType &&
                            set.PropertyType.GetGenericTypeDefinition() == typeof (Nullable<>))
                            returnType = set.PropertyType.GetGenericArguments().First();

                        try
                        {
                            value = set.PropertyType == returnType
                                        ? candidateValue
                                        : Convert.ChangeType(candidateValue, set.PropertyType);
                        }
                        catch (NullReferenceException)
                        {
                            value = null;
                        }
                    }

                    Delegate chain;
                    var changed = false;

                    var itemToUpdate = compiledChains.TryGetValue(set, out chain)
                                           ? chain.DynamicInvoke(item)
                                           : item;

                    if (itemToUpdate != null)
                    {
                        itemToUpdate.Set(set.PropertyName, value);
                        changed = true;
                    }

                    if (changed && !updates.Contains(itemToUpdate))
                        updates.Add(itemToUpdate);
                }
            }
            return updates.Count;
        }

        #endregion

        private void CompileProperties()
        {
            foreach (var set in update.PropertySets)
            {
                compiledSettersValues.Add(set, set.ValueExpression.Compile());

                var expression = set.PropertyExpression.PreviousExpressionChain();

                if (expression != null)
                    compiledChains.Add(set, expression.Compile());
            }
        }
    }
}