using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using Magiq.Providers.Sql.Support;
using MirrorMirror;

namespace Magiq.Providers.Sql.Query
{
    public class MagiqToSqlQueryable<T> : IQueryable<T>, IEntitySetAdapterProvider
    {
        private readonly IQueryable<T> baseQueryable;

        public MagiqToSqlQueryable(EntitySetAdapter entitySetAdapter)
        {
            var member = entitySetAdapter.Member;
            var table = entitySetAdapter.DataContext.GetTable(member.Association.OtherType.Type);

            EntitySetAdapter = entitySetAdapter;
            var elementType = member.Association.OtherType.Type;

            var parameter = Expression.Parameter(elementType, "x");

            var provider = (IQueryProvider) table;

            Expression body = null;

            for (var i = 0; i < member.Association.ThisKey.Count; i++)
            {
                var parentKeyValue = entitySetAdapter.Instance.Get(member.Association.ThisKey[i].Name);
                var childKeyName = member.Association.OtherKey[i].Name;
                var e = Expression.Equal(Expression.Property(parameter, childKeyName),
                                         Expression.Constant(parentKeyValue));
                body = body == null ? e : Expression.AndAlso(body, e);
            }
            var lambda = Expression.Lambda(body, parameter);
            baseQueryable = (IQueryable<T>)provider.CreateQuery(Expression.Call(typeof(Queryable),
                                                                                 "Where",
                                                                                 new[] {table.ElementType},
                                                                                 table.Expression,
                                                                                 Expression.Quote(lambda)));
        }

        #region IEntitySetAdapterProvider Members

        public EntitySetAdapter EntitySetAdapter { get; private set; }

        #endregion

        #region IQueryable<TChild> Members

        public IEnumerator<T> GetEnumerator()
        {
            return baseQueryable.GetEnumerator();
        }

        public Expression Expression
        {
            get { return baseQueryable.Expression; }
        }

        public Type ElementType
        {
            get { return baseQueryable.ElementType; }
        }

        public IQueryProvider Provider
        {
            get { return baseQueryable.Provider; }
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }

        #endregion
    }
}