using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;

namespace Magiq.Update
{
    public class Update<T> : IUpdate<T> where T : class
    {
        private readonly List<IPropertySet> propertySets;

        public Update(IQueryable<T> queryable)
        {
            Source = queryable;
            propertySets = new List<IPropertySet>();
        }

        #region IUpdate<T> Members

        public IUpdate<T> Set<TReturn>(Expression<Func<T, TReturn>> property, TReturn value)
        {
            Expression<Func<T, TReturn>> valueExpression = x => value;
            return Set(property, valueExpression);
        }

        public IUpdate<T> Set<TReturn>(Expression<Func<T, TReturn>> property, Expression<Func<T, TReturn>> value)
        {
         SetByLambdas(property, value);
            return this;
        }

        public int Execute()
        {
            if (propertySets.Count == 0)
                throw new NoPropertiesToUpdateException();

            var strategy = MagiqConfigurator.ForUpdate(this);
            var result = strategy.Execute();
            return result;
        }

        int IUpdate<T>.Update()
        {
            return Execute();
        }

        public IQueryable<T> Source { get; private set; }

        public IEnumerable<IPropertySet> PropertySets
        {
            get { return propertySets; }
        }

        public void SetByLambdas(LambdaExpression property, LambdaExpression value)
        {
            propertySets.Add(new PropertySet(property, value));
        }

        #endregion

        protected void AddPorpertySets(IEnumerable<IPropertySet> sets)
        {
            propertySets.AddRange(sets);
        }
    }
}